import time
from COMPS import Client
from COMPS.Data.WorkItem import WorkItemState

from simtools.AssetManager.FileList import FileList
from simtools.Services.Create.CreateSvc import CreateSvc
from simtools.Services.ObjectInfo.ObjectInfoSvc import ObjectInfoSvc
from simtools.Services.Run.RunSvc import RunSvc
from simtools.Services.Utility.SSMTManagers.ObjectManager import ObjectManager
from simtools.Services.Status.StatusSvc import StatusSvc
from simtools.SetupParser import SetupParser


class WorkItemManager(ObjectManager):

    def __init__(self, comps_host=None, comps_env=None, item_name="DockerWorker WorkItem", item_type="DockerWorker",
                 docker_image="ubuntu1804python3dtk", provider="COMPS", type="WI", plugin_key="1.0.0.0_RELEASE",
                 command=None, user_files=FileList(), tags={}, wo_kwargs={}):
        super().__init__(item_name=item_name, provider=provider, tags=tags)
        self.comps_host = comps_host
        self.comps_env = comps_env
        self.item_type = item_type
        self.plugin_key = plugin_key
        self.docker_image = docker_image
        self.files = user_files
        self.wo_kwargs = wo_kwargs
        self.command = command
        self.type = type

    def execute(self, check_status=True):

        self.prepare()

        self.create()

        self.run()

        self.wait_for_finish(check_status)

    def prepare(self):
        # update environments
        self.comps_host = self.comps_host or SetupParser.get('server_endpoint')
        self.comps_env = self.comps_env or SetupParser.get('environment')

    def create(self):
        # login
        Client.login(self.comps_host)

        # Create a WorkItem
        kwargs = {"item_name": self.item_name, "item_type": self.item_type, "docker_image": self.docker_image,
                  "plugin_key": self.plugin_key, "comps_env": self.comps_env, "tags": self.tags,
                  "files": self.files, "wo_kwargs": self.wo_kwargs, "command": self.command}

        self.item_id = CreateSvc.create(self.provider, self.type, kwargs)

        # Refresh local object db
        ObjectInfoSvc.create_item(type=self.type, provider=self.provider, item_id=str(self.item_id))

    def run(self):
        RunSvc.run(self.item_id)

    def status(self):
        return StatusSvc.get_status(self.item_id)

    def wait_for_finish(self, check_status=True, timeout=3600):
        if check_status:
            start = time.clock()
            state = self.status()
            states = [WorkItemState.Succeeded, WorkItemState.Failed, WorkItemState.Canceled]
            while state not in states and time.clock() - start < timeout:
                time.sleep(5)
                state = self.status()
                print('State -> {} '.format(state.name))
        else:
            print('WorkItem created in {}.'.format(self.provider))

    def add_file(self, af):
        self.files.add_asset_file(af)

    def add_wo_arg(self, name, value):
        self.wo_kwargs[name] = value

    def clear_user_files(self):
        self.files = FileList()

    def clear_wo_args(self):
        self.wo_kwargs = {}
