from simtools.Analysis.AnalyzeManager import AnalyzeManager
from simtools.Analysis.BaseAnalyzers import BaseAnalyzer
import os


class DockerPopulationAnalyzer(BaseAnalyzer):
    def __init__(self):
        super().__init__(filenames=['output\\InsetChart.json'])

    def select_simulation_data(self, data, simulation):
        # Apply is called for every simulations included into the experiment
        # We are simply storing the population data in the pop_data dictionary
        return data[self.filenames[0]]["Channels"]["Statistical Population"]["Data"]

    def finalize(self, all_data):
        import matplotlib
        matplotlib.use("Agg")
        import matplotlib.pyplot as plt

        fig = plt.figure()
        for pop in list(all_data.values()):
            plt.plot(pop)
        plt.legend([s.id for s in all_data.keys()])
        plt.show()
        plt.savefig(os.path.join(os.getcwd(), 'Population.png'))
        # plt.savefig(os.path.join('output', 'Population.png'))       # No such file or directory: 'output/Population.png'
        plt.close(fig)


analyzers =[DockerPopulationAnalyzer()]

# This code will analyze the latest experiment ran with the PopulationAnalyzer
if __name__ == "__main__":
    am = AnalyzeManager('latest', analyzers=DockerPopulationAnalyzer())
    am.analyze()