import json
import os
import pickle
from simtools.SimulationCreator.COMPSSimulationCreator import COMPSSimulationCreator
from COMPS.Data import SimulationFile


class PythonSimulationCreator(COMPSSimulationCreator):
    def __init__(self, config_builder, initial_tags,  work_queue, experiment, cache, save_semaphore, comps_experiment):
        super(PythonSimulationCreator, self).__init__(config_builder, initial_tags,  work_queue, experiment, cache, save_semaphore, comps_experiment)

    def process(self):
        self.pre_creation()

        for batch in iter(self.work_queue.get, None):
            config = {}
            for mod_fn_list in batch:
                cb = pickle.loads(pickle.dumps(self.config_builder, protocol=pickle.HIGHEST_PROTOCOL))

                # modify next simulation according to experiment builder
                # also retrieve the returned metadata
                tags = self.initial_tags.copy() if self.initial_tags else {}
                for func in mod_fn_list:
                    md = func(cb)
                    tags.update(md)
                    config.update(md)

                # Prepare the assets assets
                cb.assets.prepare(cb)

                # Create the simulation
                s = self.create_simulation(cb)

                #########################################

                # working_directory = s.sim_dir
                name = "config.json"
                # filename = os.path.join(working_directory, '%s' % name)
                # content = "{'a': 1}"
                content = json.dumps(config, sort_keys=True)
                # with open(filename, 'w') as f:
                #     f.write(content)

                s.add_file(simulationfile=SimulationFile(name, 'input'), data=str.encode(content, 'utf-8'))

                ##########################################

                # Append the environment to the tag and add the default experiment tags if any
                self.set_tags_to_simulation(s, tags, cb)

                # Add the files
                self.add_files_to_simulation(s, cb)

                # Add to the created simulations array
                self.created_simulations.append(s)

            self.process_batch()
            self.post_creation()


